package net.lemnik.eodsql.impl;

import java.util.Iterator;

import net.lemnik.eodsql.Update;
import net.lemnik.eodsql.spi.Context;

/**
 * Created on 2009/05/23
 * @author Bernd Rinn
 */
final class BatchContextProvider implements Iterable<Context<Update>> {
    private final BatchParameterKinds batchParameterKinds;
    private final Context<Update> context;
    
    BatchContextProvider(BatchParameterKinds batchParameterKinds, Context<Update> context) {
        this.batchParameterKinds = batchParameterKinds;
        this.context = context;
    }
    
    public Iterator<Context<Update>> iterator() {
        return new Iterator<Context<Update>>() {
            final Iterator<?>[] batchIterators = new Iterator<?>[batchParameterKinds.getSizeBatchParameters()];
            final Iterator<?>[] iterators = new Iterator<?>[batchParameterKinds.getSize()];
            {
                int j = 0;
                for (int i = 0; i < iterators.length; ++i) {
                    final BatchParameterKind kind = batchParameterKinds.get(i); 
                    iterators[i] = kind.getIterator(context.getParameters()[i]);
                    if (kind != BatchParameterKind.OTHER)
                    {
                        batchIterators[j++] = iterators[i];
                    }
                }
            }
            public boolean hasNext() {
                boolean hasNext = batchIterators[0].hasNext();
                for (int i = 1; i < batchIterators.length; ++i) {
                    if (batchIterators[i].hasNext() != hasNext) {
                        throw new IllegalStateException("Batch update: iterators disagree on batch size.");
                    }
                }
                return hasNext;
            }

            public Context<Update> next() {
                final Object[] parameters = new Object[iterators.length];
                for (int i = 0; i < parameters.length; ++i) {
                    parameters[i] = iterators[i].next();
                }
                return new Context<Update>(context.getAnnotation(), parameters);
            }

            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }
}
